#include "ISO7816-card.h"
#include "common.h"
#include <util/delay.h>
#include <string.h>

//ATMega Card
//ATR: 3B8F014156522049534F3738313620302E31B9
//Query: 00C101FE3E00000267016400C101FE3E00000267016400C101FE3E000002670164

//Java Card
//ATR: 3BFA1800FF8131FE454A434F5032315632333165
//Query: FF1118F6FF1118F6F8FF38F8E098E0003BFA1800FF8131FE454A434F5032315632333165
static apdu_handler_t handler;

//                  TS    T0     TA1   TB1   TC1   TD1(T=1)
ISO7816_ATR atr = {0x3B, 0x9F, {{0x11, 0x00, 0x00, 0x01},
//                               TA2   TB2   TC2   TD2 (unused)
                                {0x00, 0x00, 0x00, 0x00},
//                               TA3   TB3   TC3   TD3 (unused)
                                {0x00, 0x00, 0x00, 0x00},
//                               TA4   TB4   TC4   TD4 (unused)
                                {0x00, 0x00, 0x00, 0x00},
//                               TA5   TB5   TC5   TD5 (unused)
                                {0x00, 0x00, 0x00, 0x00}},
                   //Historical Bytes
                   {0x41,0x56,0x52,0x20,0x49,0x53,0x4f,0x37,0x38,0x31,0x36,0x20,0x30,0x2e,0x31},
                   0}; //Checksum

// unsigned char atr[] = {0x3B, 0xFA, 0x18, 0x00, 0xFF , 0x81 , 0x31 , 0xFE , 0x45 , 0x4A , 0x43 , 0x4F , 0x50 , 0x32 , 0x31 , 0x56 , 0x32 , 0x33 , 0x31, 0x65};

uint16_t ISO7816_receive_buffer(uint8_t* buffer, uint8_t length);
uint8_t  ISO7816_transmit_buffer(uint8_t* buffer, uint8_t length);
uint8_t  ISO7816_transmit_t1_header(ISO7816_T1_Header* t1_header);

void ISO7816_init(void){
	UART_init();
}
/*
void ISO7816_sendATR(void){
	UART_transmit_init();
	for (uint8_t i = 0; i < sizeof(atr); i++){
		UART_transmit_char(atr[i]);
	}
	UART_recv_init();
}*/

void ISO7816_sendATR(void){
	uint8_t checksum = 0;
	UART_transmit_init();
	UART_transmit_char(atr.initial_char);
	checksum ^= atr.t0; UART_transmit_char(atr.t0);
	
	uint8_t num_hist_bytes = NUM_HIST_BYTES(atr);
	uint8_t next_interface_bytes = INTERFACE_BYTE_SET(atr, 0);
	uint8_t i = 0;
	while (next_interface_bytes){
		if (next_interface_bytes & TA) {checksum ^= atr.interface_bytes[i].ta; UART_transmit_char(atr.interface_bytes[i].ta);}
		if (next_interface_bytes & TB) {checksum ^= atr.interface_bytes[i].tb; UART_transmit_char(atr.interface_bytes[i].tb);}
		if (next_interface_bytes & TC) {checksum ^= atr.interface_bytes[i].tc; UART_transmit_char(atr.interface_bytes[i].tc);}
		if (next_interface_bytes & TD) {checksum ^= atr.interface_bytes[i].td; UART_transmit_char(atr.interface_bytes[i].td);}
		i++;
		next_interface_bytes = INTERFACE_BYTE_SET(atr, i);
	}
	for (i = 0; i < num_hist_bytes; i++){
		checksum ^= atr.historical_bytes[i];
		UART_transmit_char(atr.historical_bytes[i]);
	}
	UART_transmit_char(checksum);
	UART_recv_init();
// 	UART_read_char();
}

uint8_t ISO7816_data_available(void){
	return UART_data_available();
}

#define RECV_ERROR(new_status) {status = (status == ISO7816_STATUS_OK) ? new_status : status;}

uint16_t process_pts(void){
	uint8_t checksum = 0xFF; //PTSS = 0xFF, so we start with that
	uint16_t ret;
	uint8_t c;
	uint8_t pts0;
	uint8_t pts1;
	uint8_t pts2;
	uint8_t pts3;
	
	ret = UART_receive_char_timeout(ISO7816_APDU_READ_TIMEOUT);
	if (ret == UART_ERR_TIMEOUT) return ISO7816_ERR_TIMEOUT;
	pts0 = ret & 0xFF;
	checksum ^= pts0;
	
	if (pts0 & 0x10){ //PTS1 avail
		ret = UART_receive_char_timeout(ISO7816_APDU_READ_TIMEOUT);
		if (ret == UART_ERR_TIMEOUT) return ISO7816_ERR_TIMEOUT;
		pts1 = ret & 0xFF;
		checksum ^= pts1;
	}
	if (pts0 & 0x20){ //PTS2 avail
		ret = UART_receive_char_timeout(ISO7816_APDU_READ_TIMEOUT);
		if (ret == UART_ERR_TIMEOUT) return ISO7816_ERR_TIMEOUT;
		pts2 = ret & 0xFF;
		checksum ^= pts2;
	}
	if (pts0 & 0x40){ //PTS3 avail
		ret = UART_receive_char_timeout(ISO7816_APDU_READ_TIMEOUT);
		if (ret == UART_ERR_TIMEOUT) return ISO7816_ERR_TIMEOUT;
		pts3 = ret & 0xFF;
		checksum ^= pts3;
	}
	
	ret = UART_receive_char_timeout(ISO7816_APDU_READ_TIMEOUT);
	if (ret == UART_ERR_TIMEOUT) return ISO7816_ERR_TIMEOUT;
	c = ret & 0xFF;
	checksum ^= c;
	
	if (c != checksum) return ISO7816_STATUS_ERR_DATA_INVALID;
	_delay_us(300);
	
	UART_transmit_init();
	UART_transmit_char(0xFF); //PTSS
	UART_transmit_char(pts0);
	if (pts0 & 0x10) UART_transmit_char(pts1);
	if (pts0 & 0x20) UART_transmit_char(pts2);
	if (pts0 & 0x40) UART_transmit_char(pts3);
	UART_transmit_char(checksum);
	
	UART_recv_init();
	
	return ISO7816_STATUS_OK;
}

uint8_t ISO7816_read_t1_header(ISO7816_T1_Header* h, uint8_t* checksum){
	uint16_t ret;
	uint8_t c = 0;
// 	static uint8_t first = 0;
	
// 	if (first){
// 		while (c == 0x00){
// 			//NAD (Node Address, not used)
// 			ret = UART_receive_char_timeout(ISO7816_APDU_READ_TIMEOUT);
// 			if (ret == UART_ERR_TIMEOUT) return 0;
// 			c = ret & 0xFF;
// 			*checksum ^= c;
// 			if (c == 0xFF) {process_pts(); return 0;}
// 		}
// 		h->node_address = 0;
// 		
// 		first = 0;
// 	} else {
		//NAD (Node Address, not used)
		ret = UART_receive_char_timeout(ISO7816_APDU_READ_TIMEOUT);
		if (ret == UART_ERR_TIMEOUT) return 0;
		c = ret & 0xFF;
		*checksum ^= c;
		if (c == 0xFF) {process_pts(); return 0;}
		h->node_address = c;
		
		// 	Protocol control byte
		c = UART_receive_char_timeout(ISO7816_APDU_READ_TIMEOUT);
		if (ret == UART_ERR_TIMEOUT) return 0;
		c = ret & 0xFF;
		*checksum ^= c;
		
// 		first = 1;
// 	}
	
	h->protocol_control = c;
// 	if (!(seq_number && (c & 0x40))) RECV_ERROR(ISO7816_STATUS_ERR_UNKNOWN); //TODO: Better response? Or no response? This is below the APDU level
	
	//Message Length
	ret = UART_receive_char_timeout(ISO7816_APDU_READ_TIMEOUT);
	if (ret == UART_ERR_TIMEOUT) return 0;
	c = ret & 0xFF;
	*checksum ^= c;
	h->message_length = c;
	
	return 1;
}

uint16_t ISO7816_read_t1_trailer(uint8_t checksum){
	uint16_t ret;
	uint8_t c;
	
	ret = UART_receive_char_timeout(ISO7816_APDU_READ_TIMEOUT);
	if (ret == UART_ERR_TIMEOUT) return ISO7816_ERR_TIMEOUT;
	c = ret & 0xFF;
	
// 	if (c != checksum) return ISO7816_STATUS_ERR_DATA_INVALID;
	
	return 1;
}

void ISO7816_process_apdu(ISO7816_T1_Header* t1_header, uint8_t* message_buffer){
	uint16_t status = ISO7816_STATUS_OK;
	
	ISO7816_APDU_Header *apdu_header = (ISO7816_APDU_Header*)&message_buffer[0];
	uint8_t* payload = &message_buffer[APDU_PAYLOAD_OFFSET];
	uint8_t payload_length = 0;
		
	uint8_t answer_buffer[255];
	uint8_t answer_length = 0;
	uint8_t max_answer_length = 0;
	
	
	//Now about interpreting length values...
	if (t1_header->message_length < sizeof(ISO7816_APDU_Header)){
		status = ISO7816_STATUS_ERR_WRONG_LENGTH;
	} else if (t1_header->message_length == sizeof(ISO7816_APDU_Header)){
		//no APDU payload, no answer payload.
		payload_length = 0;
		max_answer_length = 0;
	} else if (t1_header->message_length == sizeof(ISO7816_APDU_Header) + 1){
		//no APDU payload, but answer payload allowed.
		payload_length = 0;
		max_answer_length = APDU_PAYLOAD_LENGTH(message_buffer);
	} else { //we have APDU payload
		payload_length = APDU_PAYLOAD_LENGTH(message_buffer);
		if (t1_header->message_length - sizeof(ISO7816_APDU_Header) - 1 == payload_length){
			//APDU Header + APDU Payload, no answer payload
			max_answer_length = 0;
		} else if (t1_header->message_length - sizeof(ISO7816_APDU_Header) - 1 == (payload_length + 1)) {
			//APDU Header + APDU Payload + answer payload
			max_answer_length = message_buffer[t1_header->message_length - 1];
		} else {
			status = ISO7816_STATUS_ERR_WRONG_LENGTH;
		}
	}
	
	
	if (status == ISO7816_STATUS_OK){
		answer_length = max_answer_length;
		status = handler(apdu_header, payload, payload_length, answer_buffer, &answer_length);
		if (answer_length > max_answer_length){
			status = ISO7816_STATUS_ERR_WRONG_LENGTH;
		}
	}
	if (status != ISO7816_STATUS_OK) answer_length = 0;
	
	t1_header->node_address = 0;
	t1_header->message_length = answer_length+2;
	
	memcpy(message_buffer, answer_buffer, answer_length);
	message_buffer[answer_length]   = status >> 8;
	message_buffer[answer_length+1] = status & 0xFF;
}

uint8_t ISO7816_process(void){
	uint16_t ret;
	
	ISO7816_T1_Header t1_header;
	
	uint8_t ok;
	uint8_t checksum = 0;
	
	uint8_t buffer[255];
	
	ok = ISO7816_read_t1_header(&t1_header, &checksum);                 //T1 Protocol header
	if (!ok) return 0;
	
	ret = ISO7816_receive_buffer(buffer, t1_header.message_length); //Message Body (e.g. APDU)
	if (ret == UART_ERR_TIMEOUT) return 0;
	checksum ^= ret & 0xFF;
	
	ok = ok && ISO7816_read_t1_trailer(checksum); //T1 Protocol trailer (checksum)
	
	//------------ Process message ------------
	if (ok){
		if (t1_header.protocol_control == ISO7816_PCB_IFS_REQUEST){
			t1_header.protocol_control = ISO7816_PCB_IFS_RESPONSE;
			t1_header.node_address = 0;
			
		} else if (ISO7816_PCM_IS_I_BLOCK(t1_header.protocol_control)){
			ISO7816_process_apdu(&t1_header, buffer);
		} else {
			ok = 0;
		}
	}
	
// 	PORTB |= (1 << 7);
	
	if (!ok) return 0;
	
	//------------ Send Answer ------------
	_delay_us(200);
	PORTB &= ~(1 << 7);
	
	UART_transmit_init();
	_delay_us(200);
	
	checksum = 0;
	checksum ^= ISO7816_transmit_t1_header(&t1_header);                    //Header
	checksum ^= ISO7816_transmit_buffer(buffer, t1_header.message_length); //Payload
	UART_transmit_char(checksum);                                          //Checksum
	
	UART_recv_init();  //switch back to receive mode.
	
	return ok;
}

uint8_t ISO7816_transmit_t1_header(ISO7816_T1_Header* t1_header){
	//Node Address TODO: WTF is that? do i need to care?
	uint8_t checksum = t1_header->node_address;
	UART_transmit_char(t1_header->node_address);
	
	//Protocol control byte
	checksum ^= t1_header->protocol_control; //seq_number ? 0x40 : 0x00;;
	UART_transmit_char(t1_header->protocol_control);
	
	//Answer length (answer payload + 2 byte Status)
// 	t1_header.message_length = answer_length + 2;
	checksum ^= t1_header->message_length;
	UART_transmit_char(t1_header->message_length);
	
	return checksum;
}

uint8_t ISO7816_transmit_buffer(uint8_t* buffer, uint8_t length){
	uint8_t checksum = 0;
	
	for (uint8_t i = 0; i < length; i++){
		UART_transmit_char(buffer[i]);
		checksum ^= buffer[i];
	}
	return checksum;
}

uint16_t ISO7816_receive_buffer(uint8_t* buffer, uint8_t length){
	uint8_t c;
	uint8_t checksum = 0;
	uint16_t ret;
	
	for (uint8_t i = 0; i < length; i++){
		ret = UART_receive_char_timeout(ISO7816_APDU_READ_TIMEOUT);
		if (ret == UART_ERR_TIMEOUT) return ISO7816_ERR_TIMEOUT;
		c = ret & 0xFF;
		checksum ^= c;
		if (buffer) buffer[i] = c;
	}
	return checksum;
}

void ISO7816_set_apdu_handler(apdu_handler_t h){
	handler = h;
}