#pragma once

#include "software_serial.h"
#include "../common/ISO7816-common.h"

#define ISO7816_APDU_READ_TIMEOUT 3000 //300ms
#define ISO7816_ERR_TIMEOUT 0xFFFF

#define ISO7816_STATUS_OK                                0x9000
#define ISO7816_STATUS_ERR_INCORRECT_P1P2                0x6A86
#define ISO7816_STATUS_ERR_WRONG_P1P2                    0x6B00
#define ISO7816_STATUS_ERR_CORRECT_LENGTH_00             0x6C00
#define ISO7816_STATUS_ERR_INS_NOT_SUPPORTED             0x6D00
#define ISO7816_STATUS_ERR_CLA_NOT_SUPPORTED             0x6E00
#define ISO7816_STATUS_ERR_BYTES_REMAINING_00            0x6100
#define ISO7816_STATUS_ERR_WRONG_LENGTH                  0x6700
#define ISO7816_STATUS_ERR_SECURITY_STATUS_NOT_SATISFIED 0x6982
#define ISO7816_STATUS_ERR_FILE_INVALID                  0x6983
#define ISO7816_STATUS_ERR_DATA_INVALID                  0x6984
#define ISO7816_STATUS_ERR_CONDITIONS_NOT_SATISFIED      0x6985
#define ISO7816_STATUS_ERR_COMMAND_NOT_ALLOWED           0x6986
#define ISO7816_STATUS_ERR_WRONG_DATA                    0x6A80
#define ISO7816_STATUS_ERR_FUNC_NOT_SUPPORTED            0x6A81
#define ISO7816_STATUS_ERR_FILE_NOT_FOUND                0x6A82
#define ISO7816_STATUS_ERR_RECORD_NOT_FOUND              0x6A83
#define ISO7816_STATUS_ERR_APPLET_SELECT_FAILED          0x6999
#define ISO7816_STATUS_ERR_UNKNOWN                       0x6F00

struct ISO7816_T1_Header{
	uint8_t node_address;
	uint8_t protocol_control;
	uint8_t message_length;
} typedef ISO7816_T1_Header;

#define ISO7816_PCM_IS_I_BLOCK(pcb) (((pcb) & 0x80) == 0)
#define ISO7816_PCB_IFS_REQUEST 0xC1
#define ISO7816_PCB_IFS_RESPONSE 0xE1

#define MIN(a,b) (((a) < (b)) : (a) : (b))

//uint16_t ISO7816_handle_adpu(ISO7816_APDU_Header* header, uint8_t* payload, uint8_t payload_length, uint8_t* answer_buffer, uint8_t* answer_length);

typedef uint16_t(*apdu_handler_t)(ISO7816_APDU_Header*, uint8_t*, uint8_t, uint8_t*, uint8_t*);

void ISO7816_init(void);
void ISO7816_sendATR(void);
uint8_t ISO7816_data_available(void);
uint8_t ISO7816_process(void);
void ISO7816_set_apdu_handler(apdu_handler_t handler);


