#include <avr/eeprom.h>
#include "../common/card_common.h"

/**
 * Returns non-zero result if the keystore is empty.
 * Returns zero if at least one key has been stored.
 */
uint8_t keystore_is_empty(void);

/**
 * Initializes the "primary" key of the keystore
 */
void keystore_init_key(uint8_t* key);

/**
 * Loads a key to the keybuffer and returns the pointer to it. The pointer is valid until the next call to a keystore_ method.
 * @param secondary  0 - load the primary key, 1 - load the secondary key
 */
uint8_t* keystore_get_key(int8_t secondary);

/**
 * Update a key with the given key_delta. 
 * @param new_key_delta bitwise XOR between the old key and the given key_delta is performed and the result is stored.
 * @param update_secondary 0 - update the primary key (and mark it as primary) 1 - update the secondary key (and mark it as primary)
 */
void keystore_update_key(uint8_t* new_key_delta, uint8_t update_secondary);

/**
 * Delete all keys from the keystore
 */
void keystore_clear(void);