#include "common.h"
#include <avr/io.h>
#include <util/delay_basic.h>
#include <util/delay.h>
#include <avr/interrupt.h>

//Serial Data Line: PD6  (ICP)

#define BAUD 9600
#define SWSERIAL_PARITY EVEN
#define SWSERIAL_STOPBITS 2
#define SWSERIAL_PORT_NAME D
#define SWSERIAL_LINE 6

#define UART_ERR_TIMEOUT 0xFFFF

#ifndef XPASTE
	//Macro magic to glue #defined names together. See http://groups.google.com/group/comp.lang.c/msg/8c085fe5a8ad1296?hl=en
	#define PASTE(a,b) a ## b
	#define XPASTE(a,b) PASTE(a,b)
#endif

#define EVEN 1
#define ODD  2
#define NONE 0
#ifndef SWSERIAL_PARITY
	#define SWSERIAL_PARITY NONE
#endif
#ifndef SWSERIAL_STOPBITS
	#define SWSERIAL_STOPBITS 1
#endif

#define BIT_LENGTH 1000000/BAUD //(µSec) Duration of a RS232 Bit

#define SWSERIAL_PORT_DR XPASTE(DDR, SWSERIAL_PORT_NAME)
#define SWSERIAL_PORT    XPASTE(PORT,SWSERIAL_PORT_NAME)
#define SWSERIAL_PIN     XPASTE(PIN, SWSERIAL_PORT_NAME)

#define WRITE(x) if ((x) != 0) {SWSERIAL_PORT |= (1 << SWSERIAL_LINE);} else {SWSERIAL_PORT &= (unsigned char)~(1 << SWSERIAL_LINE);}


#define INST_LENGTH(count) ((1000000*count)/F_CPU)  //(µSec) Duration of count CPU Instructions

void UART_transmit_char(unsigned char data);

void UART_transmit_buffer_sz(char* buffer);
void UART_transmit_byte_hex(unsigned char b);

void UART_init(void);

void UART_transmit_init(void);
void UART_recv_init(void);

void UART_transmit_char(unsigned char x);

uint8_t UART_data_available(void);
uint8_t UART_read_char(void);
uint16_t UART_receive_char_timeout(uint16_t timeout);