#include "interface.h"
#undef BAUD
#undef STOPBITS
#undef PARITY
#undef USART_NAME
#define USART_NUMBER 0
// #define USART_USE_AS_STDOUT
#define USART_NAME HOST
#define BAUD 2400
#include "../common/serial.h"

#define READ_TIMEOUT 30000

uint8_t verify_hash(KEY keydelta, HASH ref_hash);

void interface_init(){
	HOST_init();
	HOST_trash_receive_buffer();
}

uint8_t interface_ping(void){
	HOST_trash_receive_buffer();
	HOST_transmit_char(IF_CMD_PING);
	return HOST_read_char_timeout(2000) != ERR_TIMEOUT;
}

uint8_t interface_query_salt(SALT salt){
	HOST_trash_receive_buffer();
	HOST_transmit_char(IF_CMD_QUERY_SALT);
	uint16_t res = HOST_read_char_timeout(READ_TIMEOUT);
	if (res != IF_RES_CMD_OK) return 0;
	return HOST_read_buffer_timeout(salt, sizeof(SALT), READ_TIMEOUT) == sizeof(SALT);
}

uint8_t interface_verify_key(HASH keyhash, SALT cardchallange, KEY newkeydelta, HASH challangeresponse){
	HASH newkeydelta_hash;
	
	HOST_trash_receive_buffer();
	HOST_transmit_char(IF_CMD_VERIFY_KEY);
	HOST_transmit_buffer(keyhash, sizeof(HASH));
	HOST_transmit_buffer(cardchallange, sizeof(SALT));
	uint16_t res = HOST_read_char_timeout(READ_TIMEOUT);
	if (res == IF_RES_CMD_OK){
		if (HOST_read_buffer_timeout(newkeydelta, sizeof(KEY), READ_TIMEOUT) != sizeof(KEY)) return 0;
		if (HOST_read_buffer_timeout(newkeydelta_hash, sizeof(HASH), READ_TIMEOUT) != sizeof(HASH)) return 0;
		if (HOST_read_buffer_timeout(challangeresponse, sizeof(HASH), READ_TIMEOUT) != sizeof(HASH)) return 0;
		if (verify_hash(newkeydelta, newkeydelta_hash)){
			return 1;
		} else {
			memset(newkeydelta, 0, sizeof(KEY));
			return 0;
		}
	} else {
		return 0;
	}
}
uint8_t interface_finalize_transaction(){
	HOST_trash_receive_buffer();
	HOST_transmit_char(IF_CMD_FINALIZE);
	return HOST_read_char_timeout(READ_TIMEOUT) == IF_RES_CMD_OK;
}
void interface_abort_transaction(){
	HOST_transmit_char(IF_CMD_ABORT);
	HOST_flush();
}

uint8_t verify_hash(KEY keydelta, HASH ref_hash){
	HASH real_hash;
	sha1(&real_hash, keydelta, sizeof(KEY)*8);
	return memcmp(real_hash, ref_hash, sizeof(HASH)) == 0;
}