
#include "lcd.h"

#ifndef XPASTE
	//Macro magic to glue #defined names together. See http://groups.google.com/group/comp.lang.c/msg/8c085fe5a8ad1296?hl=en
	#define PASTE(a,b) a ## b
	#define XPASTE(a,b) PASTE(a,b)
#endif

#ifdef LCD_8BIT_MODE
	#define LCD_CONTROL_PORT_REG    XPASTE(PORT, LCD_CONTROL_PORT_NAME)
	#define LCD_CONTROL_PORT_DR_REG XPASTE(DDR,  LCD_CONTROL_PORT_NAME)
	#define LCD_DATA_PORT_REG       XPASTE(PORT, LCD_DATA_PORT_NAME)
	#define LCD_DATA_PORT_INPUT_REG XPASTE(PIN,  LCD_DATA_PORT_NAME)
	#define LCD_DATA_PORT_DR_REG    XPASTE(DDR,  LCD_DATA_PORT_NAME)
	
	
	#define LCD_BIT_BUSY      (1 << 7)
	
	#define LCD_COMMAND_INIT ((1 << 5) | (1 << 4) | (1 << 3)) //00111000 5: Command Function set, 4: 1 for 8BIT 3: two lines, !2: 5x7font
#endif

#ifdef LCD_4BIT_MODE
	#define LCD_PORT_REG    XPASTE(PORT, LCD_PORT_NAME)
	#define LCD_PORT_DR_REG XPASTE(DDR, LCD_PORT_NAME)
	
	
	#define LCD_COMMAND_INIT ((1 << 5) | (1 << 3)) //00101000 5: Command Function set, 4: 0 for 4BIT 3: two lines, !2: 5x7font
#endif

#ifdef LCD_SERIAL_MODE
	#define LCD_PORT_REG    XPASTE(PORT, LCD_PORT_NAME)
	#define LCD_PORT_DR_REG XPASTE(DDR,  LCD_PORT_NAME)
	
	#define LCD_LINE_DATA_MODE 1
	
	#define LCD_COMMAND_INIT ((1 << 5) | (1 << 3)) //00101000 5: Command Function set, !4: 0 for 4BIT 3: two lines, !2: 5x7font
#endif

volatile uint8_t lcd_backlight = 0;

#define LCD_COMMAND_ON ((1 << 3) | (1 << 2)) //00001110 3:Command Display ON/OFF, 2: Display ON, !1: Cursor off, !0: Not blinking cursor

#define LCD_COMMAND_ON_CURSOR ((1 << 3) | (1 << 2) | (1 << 1))

#define LCD_COMMAND_CLEAR (1 << 0)
#define LCD_COMMAND_HOME (1 << 1)

#define LCD_COMMAND_SETPOS (1 << 7)

#define LCD_COMMAND_SET_CG_RAM_ADDRESS (1 << 6)

//LCD Datasheet page 6 says the max data delay is 320ns
//With the AVR at 8MHz a clock cycle is 125ns, thus 3 nops (375ns) should last.
//However, it seems like this is not always the case, so we use four nops. Better save than sorry.
#define WAIT_DATA_DELAY() asm volatile ("nop" "\n" "nop" "\n" "nop" "\n" "nop" "\n") 

#ifdef LCD_8BIT_MODE
	uint8_t lcd_read(void){
		uint8_t c;
		LCD_DATA_PORT_DR_REG = 0; //switch DATA port to input mode
		LCD_CONTROL_PORT_REG = LCD_LINE_READ | lcd_backlight;
		LCD_CONTROL_PORT_REG =  LCD_LINE_READ | LCD_LINE_STROBE | lcd_backlight;
		WAIT_DATA_DELAY();  
		c = LCD_DATA_PORT_INPUT_REG;
		LCD_CONTROL_PORT_REG = LCD_LINE_READ | lcd_backlight;
		return c;
	}

	void lcd_wait_busy(void){
		uint8_t timeout = 255;
		while ((lcd_read() & LCD_BIT_BUSY) && (timeout != 0)) {_delay_us(10); timeout--;}
		LCD_DATA_PORT_DR_REG = 0xFF; //switch DATA port to output mode again
// 		_delay_ms(2);
	}
	
	static void lcd_write_nowait(uint8_t control, uint8_t data){	
		LCD_CONTROL_PORT_REG = control | lcd_backlight;
		LCD_CONTROL_PORT_REG = control | LCD_LINE_STROBE | lcd_backlight;
		LCD_DATA_PORT_REG = data;
		LCD_CONTROL_PORT_REG = control | lcd_backlight;
	}
#endif

#ifdef LCD_4BIT_MODE
	void lcd_wait_busy(void){
 		_delay_ms(3);
	}
	
	static void lcd_write_nowait(uint8_t control, uint8_t data){	
		LCD_PORT_DR_REG = 0xFF;
		LCD_PORT_REG = control | lcd_backlight;
		LCD_PORT_REG = control | lcd_backlight | (((data >> 4) & 0xFF) << LCD_DATA_OFFSET) | LCD_LINE_STROBE;
		LCD_PORT_REG = control | lcd_backlight | (((data >> 4) & 0xFF) << LCD_DATA_OFFSET);
		//LCD_PORT_REG = control | lcd_backlight | ((data & 0x0F) << LCD_DATA_OFFSET);
		LCD_PORT_REG = control | lcd_backlight | ((data & 0x0F) << LCD_DATA_OFFSET) | LCD_LINE_STROBE;
		LCD_PORT_REG = control | lcd_backlight | ((data & 0x0F) << LCD_DATA_OFFSET);
		LCD_PORT_REG = lcd_backlight;
	}
#endif

#ifdef LCD_SERIAL_MODE
	void lcd_wait_busy(void){
		_delay_ms(2);
	}

	void lcd_write_nibble(uint8_t control, uint8_t data){	
		
		//Strobe ("E") toggle to write.
		LCD_PORT_REG |= LCD_LINE_STROBE;
		
		//Q4..7
		for (uint8_t i = 0; i < 4; i++){
			LCD_PORT_REG &= (uint8_t)~LCD_LINE_CLK;
			if (data & (1 << i)) {LCD_PORT_REG |= LCD_LINE_DATA;} else {LCD_PORT_REG &= (uint8_t)~LCD_LINE_DATA;}
			LCD_PORT_REG |= LCD_LINE_CLK;
		}
		
		//Q3 (NOT Connected)
		LCD_PORT_REG &= (uint8_t)~LCD_LINE_CLK;
		_delay_us(1);
		LCD_PORT_REG |= LCD_LINE_CLK;
		
		//Q2 R/W
		LCD_PORT_REG &= (uint8_t)~LCD_LINE_CLK;
		LCD_PORT_REG &= (uint8_t)~LCD_LINE_DATA; //Low -> Write (what we do here)
		_delay_us(1);
		LCD_PORT_REG |= LCD_LINE_CLK;
		
		//Q1 RS
		LCD_PORT_REG &= (uint8_t)~LCD_LINE_CLK;
		if (control) {LCD_PORT_REG |= LCD_LINE_DATA;} else {LCD_PORT_REG &= (uint8_t)~LCD_LINE_DATA;}
		LCD_PORT_REG |= LCD_LINE_CLK;
		
		//Q0 Backlight
		LCD_PORT_REG &= (uint8_t)~LCD_LINE_CLK;
		if (lcd_backlight) {LCD_PORT_REG |= LCD_LINE_DATA;} else {LCD_PORT_REG &= (uint8_t)~LCD_LINE_DATA;}
		_delay_us(1);
		LCD_PORT_REG |= LCD_LINE_CLK;
		
		
		_delay_us(1);
		LCD_PORT_REG &= (uint8_t)~LCD_LINE_STROBE;
		_delay_us(1);
	}

	void lcd_write_nowait(uint8_t control, uint8_t data){	
		lcd_write_nibble(control, data >> 4);
		lcd_write_nibble(control, data &  0x0F);
	}

#endif


void lcd_write(uint8_t control, uint8_t data){
	lcd_write_nowait(control, data);
	lcd_wait_busy();
}


void lcd_init(void){
	#ifdef LCD_8BIT_MODE
		LCD_DATA_PORT_DR_REG = 0xFF;
		LCD_CONTROL_PORT_DR_REG |= (LCD_LINE_DATA_MODE | LCD_LINE_READ | LCD_LINE_STROBE | LCD_BACKLIGHT);
	#endif
	#ifdef LCD_4BIT_MODE
		LCD_PORT_REG = 0;
		LCD_PORT_DR_REG = 0xFF;
	#endif
	#ifdef LCD_SERIAL_MODE
		LCD_PORT_DR_REG |= (LCD_LINE_CLK | LCD_LINE_DATA | LCD_LINE_STROBE);
	#endif
	
	_delay_ms(50);
	lcd_write_nowait(0, LCD_COMMAND_INIT);
	_delay_ms(5); //min 4.1 ms wait time, Datasheet Page 21
	lcd_write_nowait(0, LCD_COMMAND_INIT); //init for the second time, according to datasheet.
	_delay_us(110); //
	lcd_write(0, LCD_COMMAND_ON);
	_delay_ms(2);
	lcd_write(0, LCD_COMMAND_CLEAR);
	_delay_ms(20);
}

void lcd_set_backlight(uint8_t on){
	#ifdef LCD_8BIT_MODE
		if (on){
			lcd_backlight = LCD_BACKLIGHT;
			LCD_CONTROL_PORT_REG |= LCD_BACKLIGHT;
		} else {
			lcd_backlight = 0;
			LCD_CONTROL_PORT_REG &= ~LCD_BACKLIGHT;
		}
	#endif
	#ifdef LCD_4BIT_MODE
		if (on){
			lcd_backlight = LCD_BACKLIGHT;
			LCD_PORT_REG |= LCD_BACKLIGHT;
		} else {
			lcd_backlight = 0;
			LCD_PORT_REG &= ~LCD_BACKLIGHT;
		}
	#endif
}

void lcd_set_cursor(uint8_t on){
	if (on){
		lcd_write(0, LCD_COMMAND_ON_CURSOR);
	} else {
		lcd_write(0, LCD_COMMAND_ON);
	}
}

void lcd_home(void){
	lcd_write(0, LCD_COMMAND_HOME);
}

void lcd_clear(void){
	lcd_write(0, LCD_COMMAND_CLEAR);
	_delay_ms(4);
}

void lcd_set_pos(uint8_t x, uint8_t y){
	lcd_write(0, LCD_COMMAND_SETPOS | (x + 0x40 * y) );
}

void lcd_define_char(uint8_t ichar, uint8_t linedata[8]){
	for (unsigned int iline = 0; iline < 8; iline++){
		lcd_write(0, LCD_COMMAND_SET_CG_RAM_ADDRESS | ichar << 3 | iline);
		lcd_write(LCD_LINE_DATA_MODE, linedata[iline]);
	}
	lcd_write(0, LCD_COMMAND_SETPOS | 0);
}

void lcd_write_char(char c){
	lcd_write(LCD_LINE_DATA_MODE, c);
}

void lcd_fill_char(char c, uint8_t count){
	for (uint8_t i = 0; i < count; i++){
		lcd_write(LCD_LINE_DATA_MODE, c);
	}
}


void lcd_write_string(char s[], uint8_t length){
	uint8_t i;
	for (i = 0; i < length; i++){
		lcd_write_char(s[i]);
	}
}

uint8_t lcd_write_string_sz(char* s){
	uint8_t count = 0;
	while (*s != 0){
		lcd_write(LCD_LINE_DATA_MODE, *s);
		s++;
		count++;
	}
	return count;
}
uint8_t lcd_write_string_sz_P(char* s){
	uint8_t count = 0;
	char c = pgm_read_byte(s);
	while (c != 0){
		lcd_write(LCD_LINE_DATA_MODE, c);
		s++;
		count++;
		c = pgm_read_byte(s);
	}
	return count;
}

uint8_t lcd_write_string_sz_limit(char* s, uint8_t limit){
	for (uint8_t i = 0; i < limit; i++){
		if (s[i] == 0) return i;
		lcd_write(LCD_LINE_DATA_MODE, s[i]);
	}
	return limit;
}

uint8_t lcd_write_byte(uint8_t b){
	char buffer[4]; //"255" is 3 chars + \0 char
	itoa(b,buffer, 10);
	return lcd_write_string_sz(buffer);
}
void  lcd_write_byte_hex(uint8_t b){
	char buffer[2]; //"FF" is 2 chars
	uint8_t n;
	n = '0' + (b & 0xF);
	buffer[1] = (n > '9') ? n + 7 : n;
	n = '0' + (b >> 4 & 0xF);
	buffer[0] = (n > '9') ? n + 7 : n;

	lcd_write_string(buffer, 2);
}
