#include <avr/io.h>
#include <stdio.h>
#include <avr/pgmspace.h>

#define DOOR_PORT PORTC
#define DOOR_DDR  DDRC
#define DOOR_PIN  PINC

#define DOOR_PIN_UNLOCKED (1 << 4)
#define DOOR_PIN_LOCKED   (1 << 5)
#define DOOR_PIN_OPEN     (1 << 6)
#define DOOR_PIN_ALARM    (1 << 7)
#define DOOR_PIN_MASK     (DOOR_PIN_UNLOCKED | DOOR_PIN_LOCKED | DOOR_PIN_OPEN | DOOR_PIN_ALARM)
#define DOOR_PIN_FLIP     (DOOR_PIN_UNLOCKED | DOOR_PIN_LOCKED) //these pins are active low, flip the values after reading
#define DOOR_PIN_OFFSET 4 //shifting the above PIN bits by this offet, gets converts them to the STATUS flags below

#define DOOR_STATUS_UNLOCKED   (1 << 0)
#define DOOR_STATUS_LOCKED     (1 << 1)
#define DOOR_STATUS_OPEN       (1 << 2)
#define DOOR_STATUS_ALARM_CUT  (1 << 3)
#define DOOR_STATUS_HW_MASK    (DOOR_STATUS_UNLOCKED | DOOR_STATUS_LOCKED | DOOR_STATUS_OPEN | DOOR_STATUS_ALARM_CUT)
#define DOOR_STATUS_ALARM      (1 << 4)
#define DOOR_STATUS_UNLOCKING  (1 << 5)
#define DOOR_STATUS_LOCKING    (1 << 6)
#define DOOR_STATUS_SW_MASK    (DOOR_STATUS_ALARM | DOOR_STATUS_UNLOCKING | DOOR_STATUS_LOCKING)
#define DOOR_MAX_STATUS_FLAG 6

#define COMMAND_UNLOCK_PIN  (1 << 2)
#define COMMAND_LOCK_PIN    (1 << 3)

#define KEYMATIC_POWER_PORT PORTA
#define KEYMATIC_POWER_DDR  DDRA
#define KEYMATIC_POWER_LINE (1 << 7)

void door_init(void);

void door_tick(void);

uint8_t door_get_status(void);

void door_set_keymatic_charge_power(uint8_t on);
uint8_t door_clear_alarm(void);

void unlock(void);
void lock(void);
uint8_t toggle_lock_unlock( void (*msg_target)(const char*) );
uint8_t unlock_checked( void (*msg_target)(const char*) );
uint8_t lock_checked( void (*msg_target)(const char*) );
void door_write_status(void);