#include "power_monitor.h"
#include "log.h"

#define POWER_BATT_POLL_INTERVAL 100  //100*10ms = 1sec
volatile uint8_t power_batt_update_timer = 0;

#define POWER_MAX_BATT_SHUTDOWN_DISABLE_TIME 3000 //2000*10ms = 30sec
volatile uint16_t power_batt_shutdown_disable_timer = 0;

uint8_t power_status;

uint16_t main_batt_voltage;
uint16_t keymatic_batt_voltage;

void power_init(void){
	POWER_DDR &= ~(1 << POWER_PIN_AC_ON | 1 << POWER_PIN_MAIN_BATTERY_VOLTAGE | 1 << POWER_PIN_KEYMATIC_BATTERY_VOLTAGE);
}

void power_tick(void){
	if (power_batt_update_timer != 0) power_batt_update_timer--;
	
	if (power_batt_shutdown_disable_timer){
		power_batt_shutdown_disable_timer--;
		if (power_batt_shutdown_disable_timer == 0){
			POWER_DDR  &= ~(1 << POWER_PIN_AC_ON);
			POWER_PORT &= ~(1 << POWER_PIN_AC_ON);
		}
	}
}

void power_process(void){
	if (power_batt_shutdown_disable_timer == 0){ //Only check of AC pin if we don't pull it up ourself

		//Check AC Status
		uint8_t ac = (POWER_PIN & (1 << POWER_PIN_AC_ON)) != 0;
		if (ac)	{ //AC On
			if ((power_status & POWER_STATUS_AC_ON) == 0){ //Was off
				log_append(LOG_EVENT_AC_RETURN, main_batt_voltage);
				power_status |= POWER_STATUS_AC_ON;
			}
		} else { //AC off 
			if ((power_status & POWER_STATUS_AC_ON) != 0){ //was on
				log_append(LOG_EVENT_AC_FAIL, main_batt_voltage);
				power_status &= ~POWER_STATUS_AC_ON;
			}
		}
	}
	
	if (power_batt_update_timer == 0){
		power_batt_update_timer = POWER_BATT_POLL_INTERVAL;
		power_update_batt_voltage();
	}
	
}

uint8_t power_get_status(void){
	return power_status;
}

void power_disable_battery_shutdown(void){
	//Pull AC pin high to disable USV low battery shutdown
	POWER_PORT |= (1 << POWER_PIN_AC_ON);
	POWER_DDR  |= (1 << POWER_PIN_AC_ON);
	power_batt_shutdown_disable_timer = POWER_MAX_BATT_SHUTDOWN_DISABLE_TIME;
}

void power_enable_battery_shutdown(void){
	//Float AC pin high to disable USV low battery shutdown
	POWER_DDR  &= ~(1 << POWER_PIN_AC_ON);
	POWER_PORT &= ~(1 << POWER_PIN_AC_ON);
	power_batt_shutdown_disable_timer = 0;
}

void power_update_batt_voltage(void){
	main_batt_voltage = ADC_read_voltage(POWER_PIN_MAIN_BATTERY_VOLTAGE, 1420);
	keymatic_batt_voltage = ADC_read_voltage(POWER_PIN_KEYMATIC_BATTERY_VOLTAGE, 507);
	
	if (main_batt_voltage < POWER_MAIN_BATT_LOW_VOLTAGE){
		if ((power_status & POWER_STATUS_MAIN_BATT_LOW) == 0){
			log_append(LOG_EVENT_MAIN_BATTERY_LOW, main_batt_voltage);
			power_status |= POWER_STATUS_MAIN_BATT_LOW;
		}
	} else {
		power_status &= ~POWER_STATUS_MAIN_BATT_LOW;
	}
	
	if (keymatic_batt_voltage < POWER_KEYMATIC_BATT_LOW_VOLTAGE){
		if ((power_status & POWER_STATUS_KEYMATIC_BATT_LOW) == 0){
			log_append(LOG_EVENT_KEYMATIC_BATTERY_LOW, keymatic_batt_voltage);
			power_status |= POWER_STATUS_KEYMATIC_BATT_LOW;
		}
	} else {
		power_status &= ~POWER_STATUS_KEYMATIC_BATT_LOW;
	}
}

void power_write_status(void){
	printf_P(PSTR("Main Power: %04i0mV ("), main_batt_voltage);
	if (power_status & POWER_STATUS_AC_ON){
		printf_P(PSTR("On AC"));
	} else {
		printf_P(PSTR("On Battery"));
	}
	if (power_status & POWER_STATUS_MAIN_BATT_LOW) printf_P(PSTR(", LOW!"));
	
	printf_P(PSTR(")~ Keymatic: %04i0mV ("), keymatic_batt_voltage);
	if (power_status & POWER_STATUS_AC_ON){
		printf_P(PSTR("On AC"));
	} else {
		printf_P(PSTR("On Battery"));
	}
	if (power_status & POWER_STATUS_KEYMATIC_BATT_LOW) printf_P(PSTR(", LOW!"));
	
	printf_P(PSTR(")"));
}
