import java.io.File;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.util.List;

import javax.smartcardio.CardException;
import net.sourceforge.gpj.cardservices.AID;
import net.sourceforge.gpj.cardservices.GPUtil;

@SuppressWarnings("restriction")
public class Main {

	
	public static void main(String args[]) throws NoSuchAlgorithmException, CardException, IOException{
		int i = 0;
		
		JavaCardProgrammer card = null;
		//JavaCardProgrammer card = new JavaCardProgrammer();
		//writeAppletList(card.readAppletList());
		
		while(i < args.length){
		
			if (args[i].equals("--help")){
				help();
			} else if (args[i].equals("--setupcard")){
				if (card == null) card = new JavaCardProgrammer();
				card.initializeCardApplet(new File("./applet.cap"));
				writeAppletList(card.readAppletList());				
			} else if (args[i].equals("--updatecard")){
				i++;
				if (args.length < i-1) {help(); System.exit(1);}
				if (args[i].length() != 128 && args[i].length() != 255) {help(); System.exit(1);}
				byte[] readkey = GPUtil.stringToByteArray(args[1]);
				
				if (card == null) card = new JavaCardProgrammer();
				
				//Backup card key
				byte[] currentKey = card.readCardKey(readkey);
				if (currentKey == null) return;
				
				//Reset card
				card.close();
				card = new JavaCardProgrammer();

				//Update Applet
				card.initializeCardApplet(new File("./applet.cap"));
				
				byte[] readKey = card.setCardKey(currentKey);
				if (readKey != null){
					System.out.println("Applet updated successfully!");
					System.out.println("To read the (current) card key or update the card use the read key:");
					System.out.println(GPUtil.byteArrayToString(readKey).replace(" ", ""));
					System.out.println("");
				}
				
			} else if (args[i].equals("--clearkey")){
				if (card == null) card = new JavaCardProgrammer();
				if (card.clearCardKey()){
					System.out.println("Card key cleared successfully!");
					System.out.println("");
				}
				
			} else if (args[i].equals("--setkey")){
				if (card == null) card = new JavaCardProgrammer();
				i++;
				if (args.length < i-1) {help(); System.exit(1);}
				if (args[i].length() != 128) {help(); System.exit(1);}
				byte[] key = GPUtil.stringToByteArray(args[1]);
				byte[] readKey = card.setCardKey(key);
				if (readKey != null){
					System.out.println("Initial key written successfully!");
					System.out.println("To read the (current) card key or update the card use the read key:");
					System.out.println(GPUtil.byteArrayToString(readKey).replace(" ", ""));
					System.out.println("");
				}
				
			} else if (args[i].equals("--readkey")){
				if (card == null) card = new JavaCardProgrammer();
				i++;
				if (args.length < i-1) {help(); System.exit(1);}
				if (args[i].length() != 128 && args[i].length() != 255) {help(); System.exit(1);}
				byte[] readkey = GPUtil.stringToByteArray(args[1]);
				byte[] currentKey = card.readCardKey(readkey);
				if (currentKey != null){
					System.out.println("Current Key read successfully!");
					System.out.println(GPUtil.byteArrayToString(currentKey).replace(" ", ""));
					System.out.println("");
				}
				
			} else if (args[i].equals("--console")){
				Integer keyslot = 0;
				if (args.length >= i && !args[i].startsWith("--")){
					keyslot = Integer.parseInt(args[i]);
					i++;
				}
				try {
					PortalControlInterface portal = new PortalControlInterface();
					portal.shell();
					portal.close();
				} catch (PortalControlInterface.PortalControlNotFoundException ex){
					System.out.println("Portal Control not found");
				}
			} else {
				help();
				System.exit(1);
			}
			i++;
		}
		System.out.println("Bye.");
	}
	
	private static void writeAppletList(List<AID> list){
		System.out.println(list.size() + " applet(s) installed on card:");
		for (AID a : list){
			System.out.println("Applet ID: " + a.toString());
		}
	}
	
	private static void help(){
		System.out.println("Usage: ");
		System.out.println("--setupcard         : Install the Portal Applet on a Javacard. Any auth key on the card will be cleared.");
		System.out.println("--setkey <key>      : Set the Portal auth key on the card (can only be used once). The key is given as 128 hex digits (64 bytes)");
		System.out.println("--updatecard <rkey> : Update the Portal Applet on a Javacard. The auth key on the card will be backed up. You need to supply the read key that --setkey returned.");
		System.out.println("--readkey <rkey>    : Read the auth key on the card. You need to supply the read key that --setkey returned.");
		System.out.println("--clearkey          : Delete the Portal auth key from the card (only works on cards with debug commands enabled).");
	}
}
