import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.nio.charset.Charset;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import gnu.io.CommPortIdentifier;
import gnu.io.PortInUseException;
import gnu.io.SerialPort;
import gnu.io.UnsupportedCommOperationException;
import jline.*;

@SuppressWarnings("restriction")
public class PortalControlInterface {
	
	private final Charset CHARSET = Charset.forName("UTF-8");
	private final String PORTAL_PROBE_COMMAND = "\n";
	private final String PORTAL_PROBE_IDENTIFIER = "Mezu Login:";
	private final long PROBE_TIMEOUT = 300;
	private final String PORTAL_PROMPT = "> ";
	
	private boolean login = false;
	
	private SerialPort port;
	
	public PortalControlInterface() throws PortalControlNotFoundException{
		java.util.Enumeration<CommPortIdentifier> portEnum = CommPortIdentifier.getPortIdentifiers();
		while (portEnum.hasMoreElements()){
			CommPortIdentifier portIdentifier = portEnum.nextElement();
			if (portIdentifier.getPortType() == CommPortIdentifier.PORT_SERIAL){
				System.out.print("Found Serial Port: " + portIdentifier.getName());
				try{
					port = openPort(portIdentifier);
					try {
						System.out.print(" Probing...");
						if (probePort(port)){
							System.out.println(" PortalControl found!");
							break;
						} else {
							System.out.println(" negative.");
							port.close();
							port = null;
						}
					} catch (IOException ex) {
						System.out.println(" Read failed: " + ex.toString());
					}
				} catch (IOException ex) {
					System.out.println(" IO Error: " + ex.toString());
				} catch (UnsupportedCommOperationException ex){
					System.out.println(" Failed to configure: " + ex.toString());
				} catch (PortInUseException ex){
					System.out.println(" In use, skipping");
				}
			}
		}
		if (port == null) throw new PortalControlNotFoundException();
	}
	
	private SerialPort openPort(CommPortIdentifier portIdentifier) throws PortInUseException, UnsupportedCommOperationException, IOException{
		SerialPort port = (SerialPort)portIdentifier.open(this.getClass().getName(), 2000);
		try {
			port.setSerialPortParams(9600,SerialPort.DATABITS_8,SerialPort.STOPBITS_1,SerialPort.PARITY_NONE);
		} catch (UnsupportedCommOperationException ex){
			port.close();
			throw ex;
		}
		return port;
	}
	
	private boolean probePort(SerialPort port) throws IOException{
		
		while (port.getInputStream().available() != 0) port.getInputStream().read(); //trash buffer
		port.getOutputStream().write(PORTAL_PROBE_COMMAND.getBytes(CHARSET));
		Integer status = waitString(new String[]{PORTAL_PROBE_IDENTIFIER , PORTAL_PROMPT}, PROBE_TIMEOUT);
		this.login = (status != null && 1 == status);
		return status != null;
	}
	
	private void writeLine(String line) throws IOException{
		port.getOutputStream().write(line.getBytes(CHARSET));
		port.getOutputStream().write('\n');
	}
/*	private String readLine() throws IOException{
		StringWriter writer = new StringWriter();
		int c = 0;
		while ((c = this.reader.read()) != '\n'){
			writer.write(c);
		}
		return writer.toString();
	}*/
	
	/**
	 * Waits for any of the given strings to appear in the input from the
	 * serial port.
	 * @param strings
	 * @param timeout
	 * @return index of the string found, null on timeout
	 * @throws IOException
	 */
	private Integer waitString(String[] strings, long timeout) throws IOException{
		InputStreamReader reader = new InputStreamReader(port.getInputStream());
		StringWriter writer = new StringWriter();
		char[] buffer = new char[1024];
		
		long tStart = System.currentTimeMillis();
		
		while ((System.currentTimeMillis() - tStart) < timeout){
			if (!reader.ready()){
				try { Thread.sleep(10); } catch (InterruptedException e) {}
			} else {
				int count = reader.read(buffer);
				writer.write(buffer, 0, count);
				for (int i = 0; i < strings.length; i++){
					if (writer.toString().contains(strings[i])){
						return i;
					}
				}
			}
		}
		return null;
	}
	
	public void shell() throws IOException{
		final ConsoleReader  console = new ConsoleReader();
		final Pattern newKeyPattern = Pattern.compile("New key: ([0-9A-F]{128})", Pattern.CASE_INSENSITIVE | Pattern.MULTILINE);
		Matcher m = newKeyPattern.matcher("fdsfdf\nNew key: 6F7AE2ED44007E0C25015FC5BFD4B598F2AD06BF817227A21CFACBC054927C6EA2AF3BBE12704CA575FA27035B413AB1D2F59C46E9320E5505FE83C0E011084D\nfdsfds");
		m.matches();
		
		Thread readThread = new Thread(){
			public void run(){
				InputStreamReader input;
				char[] buffer = new char[512];
				StringBuffer sb = new StringBuffer(1024);
				try {
					input = new InputStreamReader(port.getInputStream());
					while (true){
						int len = input.read(buffer);
						sb.append(buffer, 0, len);
						int lineend = sb.indexOf("\n");
						if (lineend != -1) sb.replace(0, lineend+1, "");
						
						if (sb.lastIndexOf("logout\n") != -1) break;
						Matcher m = newKeyPattern.matcher(sb.toString());
						if (m.matches()){
							System.out.print("k:" + m.group(1));
							sb = new StringBuffer(1024);
						}
						
						System.out.print(new String(buffer,0,len));
						System.out.flush();
					}
					System.out.println("Exited");
				} catch (IOException e) {
					e.printStackTrace();
					System.exit(2);
				}
			}
		};
		readThread.start();
		
		writeLine("");
		
		while(readThread.isAlive()){
			int c =  console.readVirtualKey();
			port.getOutputStream().write(c);
		}
		System.out.println("Shell Exited");
	}
	
	public void close(){
		if (login){
			try {
				writeLine("logout");
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		port.close();
		port = null;
	}
	
	@SuppressWarnings("serial")
	static class PortalControlNotFoundException extends Exception{
		
	}
	
}
