#define F_CPU 1000000

#include <avr/io.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>
#define ADC_VBAT 2

#define PIN_RELAIS_OFF (1 << 0)
#define PIN_AC_ON      (1 << 1)
#define PIN_STATUS     (1 << 2)
#define PIN_RELAIS_ON  (1 << 3)
#define PIN_VBAT       (1 << 4)

#define AC_ON() (PINB & PIN_AC_ON)

uint8_t battery_on = 0;
#define BATTERY_RELAIS_ON()  {PORTB |= PIN_RELAIS_ON;  _delay_ms(5); PORTB &= ~PIN_RELAIS_ON;  _delay_ms(5);}
#define BATTERY_RELAIS_OFF() {PORTB |= PIN_RELAIS_OFF; _delay_ms(5); PORTB &= ~PIN_RELAIS_OFF; _delay_ms(5);}

#define BATTERY_TIMEOUT 20
uint8_t battery_timeout = BATTERY_TIMEOUT;
#define BATTERY_VOLTAGE_THRESHOLD 1100
#define BATTERY_VOLTAGE_MAX       1380

ISR(ADC_vect){
	;
}

void sleep(void){
	wdt_reset();
	set_sleep_mode(SLEEP_MODE_ADC);
	sleep_mode();
}

uint16_t ADC_read(uint8_t channel) {
	uint16_t result;
	
	ADMUX = channel | (1 << REFS1) | (1 << REFS2); // Select channel, 2.56V internal Reference voltage.
	
	ADCSRA = (1 << ADEN) | (1 << ADPS2) |(1 << ADIE); // | (1 << ADPS1) ; //Enable and set Prescaler /16
	
	//Warmup read
	ADCSRA |= (1 << ADSC);
	sleep();
	while (ADCSRA & (1 << ADSC)) {;}
	result = ADCW;
	result = 0; 
	
	//Actual Read
	for(uint8_t i=0; i < 4; i++){
		ADCSRA |= (1 << ADSC);  
		sleep();
		while (ADCSRA & (1 << ADSC)) {;}
		result += ADCW;
	}
	
	//Disable ADC
	ADCSRA &= ~(1 << ADEN);
	
	result /= 4;
	
	return result;
}


uint16_t ADC_read_voltage(uint8_t channel, uint16_t mvmax){
	uint32_t raw = ADC_read(channel);
	return ((raw * (uint32_t)mvmax) / 1024L);
}


int main(void){
	wdt_enable(WDTO_500MS);
	wdt_reset();
	sei();
	
	DDRB = PIN_RELAIS_OFF | PIN_RELAIS_ON;
	PRR = (1 << PRUSI) | (1 << PRTIM1) | (1 << PRTIM0); //Shutdown not needed hardware (USI and TIMER1)
	
 	while (1){
		wdt_reset();
		if (AC_ON()){
			if (! battery_on) {
				BATTERY_RELAIS_ON();
				battery_on = 1;
			}
			
			DDRB  |= PIN_STATUS;
			PORTB |= PIN_STATUS;
			
			battery_timeout = BATTERY_TIMEOUT;
		} else {
			PORTB &= ~PIN_STATUS;
			DDRB  &= ~PIN_STATUS;
			
			
			if (ADC_read_voltage(ADC_VBAT, BATTERY_VOLTAGE_MAX) <= BATTERY_VOLTAGE_THRESHOLD){
				battery_timeout--;
// 				//Battery voltage too low for too long.
// 				//Commit suicide
				if ((battery_timeout == 0) && ((PINB & PIN_STATUS) == 0)){
					BATTERY_RELAIS_OFF();
// 					//We should be dead by now, but clean the state
// 					//in case the capaciator lasts a little longer.
					battery_on = 0;
					battery_timeout = BATTERY_TIMEOUT;
				}
			} else {
				battery_timeout = BATTERY_TIMEOUT;
			}
		}
// 		_delay_ms(10);
	}
}