#pragma once
#include <avr/io.h>

typedef struct{
	uint8_t ta;
	uint8_t tb;
	uint8_t tc;
	uint8_t td;
} ISO7816_Interface_Bytes;

#define TA 0x01
#define TB 0x02
#define TC 0x04
#define TD 0x08

#define ATR_MAX_INTERFACE_BYTE_SETS 5

typedef struct ISO7816_ATR{
	uint8_t initial_char;
	uint8_t t0;
	ISO7816_Interface_Bytes interface_bytes[ATR_MAX_INTERFACE_BYTE_SETS];
	char historical_bytes[15];
	uint8_t checksum;
} ISO7816_ATR;

#define NUM_HIST_BYTES(ATR) ((ATR).t0 & 0x0F)

//Valid interface bytes in set i
#define INTERFACE_BYTE_SET(ATR, I) (((I) == 0) ? ((ATR).t0 >> 4) : ((ATR).interface_bytes[(I)-1].td >> 4))
#define ATR_PROTO(ATR,I) ((INTERFACE_BYTE_SET(ATR,I) & TD) ? (ATR).interface_bytes[I].td & 0x0F : 0)
#define ATR_FI(ATR,I) ((INTERFACE_BYTE_SET(ATR,I) & TA) ? (ATR).interface_bytes[I].ta >> 4 : 0)
#define ATR_DI(ATR,I) ((INTERFACE_BYTE_SET(ATR,I) & TA) ? (ATR).interface_bytes[I].ta & 0x0F : 0)
#define ATR_N(ATR,I) ((INTERFACE_BYTE_SET(ATR,I) & TC) ? (ATR).interface_bytes[I].tc : 0)

typedef struct ISO7816_APDU_Header{
	uint8_t class;
	uint8_t instruction;
	uint8_t param1;
	uint8_t param2;
} ISO7816_APDU_Header;

#define APDU_LENGTH_BYTE 4
#define APDU_PAYLOAD_OFFSET 5
#define APDU_PAYLOAD_LENGTH(APDU) (APDU)[APDU_LENGTH_BYTE]

#define APDU_RESULT_OK 0x9000