#include "adc.h"

uint16_t ADC_read(uint8_t mux) {
	uint8_t i;
	uint16_t result;
	
	ADMUX = mux | (1 << REFS0) | (1 << REFS1) ;                      // Select channel, 2.56V internal Reference voltage.
	
	ADCSRA = (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);    // Frequenzvorteiler 
								// setzen auf 8 (1) und ADC aktivieren (1)
	
	/* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
		also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
	ADCSRA |= (1<<ADSC);              // eine ADC-Wandlung 
	while ( ADCSRA & (1<<ADSC) ) {
		;     // auf Abschluss der Konvertierung warten 
	}
	result = ADCW;  // ADCW muss einmal gelesen werden,
					// sonst wird Ergebnis der nächsten Wandlung
					// nicht übernommen.
	
	/* Eigentliche Messung - Mittelwert aus 4 aufeinanderfolgenden Wandlungen */
	result = 0; 
	for( i=0; i<4; i++ )
	{
		ADCSRA |= (1<<ADSC);            // eine Wandlung "single conversion"
		while ( ADCSRA & (1<<ADSC) ) {
		;   // auf Abschluss der Konvertierung warten
		}
		result += ADCW;                 // Wandlungsergebnisse aufaddieren
	}
	ADCSRA &= ~(1<<ADEN);             // ADC deaktivieren (2)
	
	result /= 4;                     // Summe durch vier teilen = arithm. Mittelwert
	
	return result;
}


uint16_t ADC_read_voltage(uint8_t mux, uint16_t mvmax){
	uint32_t raw = ADC_read(mux);
	return ((raw * mvmax) / 1024);
}
