#pragma once

#include <../common/sha1/sha1.h>
#include "../common/ISO7816-common.h"
#include "../common/card_common.h"

#define CARDREADER_INTERRUPT_RESET 0
#define CARDREADER_INTERRUPT_HOST_ALIVE 1
#define CARDREADER_INTERRUPT_CARDREADER_ALIVE 2
#define CARDREADER_INTERRUPT_CARD_INSERTED 5
#define CARDREADER_INTERRUPT_CARD_REMOVED 6

#define CARDREADER_MAX_MSG_LEN 255
#define CARDREADER_MAX_MSG_BODY_LEN (CARDREADER_MAX_MSG_LEN-1)
typedef struct{
	uint8_t type;
	uint8_t body[CARDREADER_MAX_MSG_BODY_LEN];
} cardreader_message_t;

typedef struct{
	uint8_t msg_type; //type 0 = Sys Message
	uint8_t command;
	uint8_t arg;
} cardreader_sys_message_t;

#define CARDREADER_MSG_TYPE_SYS 0
#define CARDREADER_MSG_TYPE_CARD 1
#define CARDREADER_MSG_TYPE_DISPALY 2

typedef struct{
	uint8_t msg_type; //type 1 = Card Message
	ISO7816_APDU_Header header;
	uint8_t body[100]; //TODO: make dynamic, somehow
} cardreader_card_message_t;

#define CARDREADER_SYS_MSG_GET_CARD_STATUS 0
#define CARDREADER_SYS_MSG_GET_CARD_ATR 1
#define CARDREADER_SYS_MSG_IS_DISPLAY_CONNECTED 2

#define CARD_STATUS_REMOVED 0
#define CARD_STATUS_INSERTED 1
#define CARD_STATUS_READY 2
#define CARD_STATUS_ERROR 0xFF

typedef struct{
	uint8_t msg_type; //type 2 = Display Message
	uint8_t command;
	uint8_t body[48]; //TODO: make dynamic, somehow
} cardreader_display_message_t;


#define CARDREADER_DISPLAY_MSG_CLEAR 0
#define CARDREADER_DISPLAY_MSG_SET_POS 1
#define CARDREADER_DISPLAY_MSG_WRITE 2
#define CARDREADER_DISPLAY_MSG_SET_BACKLIGHT 3


#define CARD_MSG_HEADER(buffer) ((ISO7816_APDU_Header*)(buffer))
#define CARD_MSG_BODY(buffer) ((buffer)+ sizeof(ISO7816_APDU_Header))
#define CARD_MSG_BODY_LENGTH(msg_length) ((msg_length)-sizeof(ISO7816_APDU_Header))
#define CARD_RESULT_STATUS(buffer) (*((uint16_t*)(buffer)))
#define CARD_RESULT_BODY(buffer) ((buffer)+2)
#define CARD_RESULT_BODY_LENGTH(msg_length) ((msg_length)-2)
