#ifndef _I2C_MASTER_H
#define _I2C_MASTER_H

#include "i2c_common.h"

#define I2C_SUCCESS 0

#define I2C_ERROR_TIMEOUT               0x80 //wait ready timed out (note, that this is a mask, the lower bits specify the location where the timeout occured)

#define I2C_ERROR_START_SEND_FAILD        0x1 //unable to send the START condition on the bus.
#define I2C_ERROR_DEST_NOT_ACCEPTED       0x2 //NACK returnd after the destination address has been sent
#define I2C_ERROR_SEND_MESSAGE_TOO_LONG   0x5 //NACK returned after a byte has been sent, but there were still more bytes to be sent.
#define I2C_ERROR_SEND_MESSAGE_TOO_SHORT  0x6 //ACK returned after the last byte has been sent, 
#define I2C_ERROR_RECV_MESSAGE_TOO_LONG   0x7
#define I2C_ERROR_RECV_MESSAGE_TOO_SHORT  0x8

#define I2C_TIMEOUT 0xFFFF //wait ready timeout (loop iterations)

#define I2C_ERROR(ret) {i2c_master_last_error_status = (TWSR & I2C_STATUS_MASK); i2c_master_last_error_control = TWCR; i2c_master_stop(); return (ret);}

uint8_t i2c_master_last_error_status;
uint8_t i2c_master_last_error_control;
uint8_t i2c_master_last_error_position;

void i2c_master_init(void);

void i2c_master_set_speed(uint8_t bitrate);

void i2c_master_reset(void);

/**
 * Writes a buffer to a target
 * @param dest i2c address of the target host
 * @param addr register address at the target host
 * @param addr_size size of the register address (1 or 2 byte)
 * @param length number of bytes to transmit
 * @param buffer pointer to the transmit buffer
 */
uint8_t i2c_master_write(uint8_t dest, uint16_t addr, uint8_t addr_size, uint8_t* buffer, uint8_t length);

/**
 * Reads bytes from the target into a buffer
 * @param dest i2c address of the target host
 * @param addr register address at the target host
 * @param addr_size size of the register address (1 or 2 byte)
 * @param length number of bytes to read
 * @param buffer pointer to the receive buffer
 */
uint8_t i2c_master_read(uint8_t dest, uint16_t addr, uint8_t addr_size, uint8_t* buffer, uint8_t length);

#endif