#ifndef TIMER_H
#define TIMER_H

#include <avr/interrupt.h> 

#define CLOCK_SOURCE_DISABLED 0
#define CLOCK_SOURCE_CPU_CLK 1
#define CLOCK_SOURCE_CPU_CLK_DIV8 2
#define CLOCK_SOURCE_CPU_CLK_DIV64 3
#define CLOCK_SOURCE_CPU_CLK_DIV256 4
#define CLOCK_SOURCE_CPU_CLK_DIV1024 5
#define CLOCK_SOURCE_T0_PIN_FALL_EDGE 6
#define CLOCK_SOURCE_T0_PIN_RAISE_EDGE 7

#define TIMER_OVERFLOW_INTERUPT_ENABLE (1<<TOIE0)

#if   F_CPU == 20000000
	#define INTERRUPTS_PER_MS 10
	#define TIMER_INIT_VALUE 6 //initialize counter to 6. We want to count for 250 steps to get 250 msec resolution, not 256
#elif F_CPU == 16000000
	#define INTERRUPTS_PER_MS 8
	#define TIMER_INIT_VALUE 6 //initialize counter to 6. We want to count for 250 steps to get 250 msec resolution, not 256
#elif F_CPU == 8000000
	#define INTERRUPTS_PER_MS 4
	#define TIMER_INIT_VALUE 6 //initialize counter to 6. We want to count for 250 steps to get 250 msec resolution, not 256
#elif F_CPU == 4000000
	#define INTERRUPTS_PER_MS 2
	#define TIMER_INIT_VALUE 6 //initialize counter to 6. We want to count for 250 steps to get 250 msec resolution, not 256
#elif F_CPU == 2000000
	#define INTERRUPTS_PER_MS 1
	#define TIMER_INIT_VALUE 6 //initialize counter to 6. We want to count for 250 steps to get 250 msec resolution, not 256
#elif F_CPU == 1000000
	#define INTERRUPTS_PER_MS 1
	#define TIMER_INIT_VALUE 131 //initialize counter to 131. We want to count for 125 steps to get 250 msec resolution, not 256
#else
	#error "Unknown CPU Freq! Timer will not work properly"
#endif


#ifndef TCCR0
	#define TCCR0 TCCR0B //some chips (e.g. ATTiny2313 have an two TCCR0 registers, the B one is what we want)
#endif

#ifndef TIMSK
	#define TIMSK TIMSK0
#endif


extern void timer_handler(void);

volatile unsigned int timer_interrupt_counter = 0; //interrupts since last ms timer event (counts up to INTERRUPTS_PER_MS)
volatile unsigned int timer_interval;

void timer_init(unsigned int interval){
	timer_interval = interval * INTERRUPTS_PER_MS;
	
	TCCR0 = CLOCK_SOURCE_CPU_CLK_DIV8; //clock source is CPU clock, count every 8 cycles.
	TCNT0 = TIMER_INIT_VALUE;
	timer_interrupt_counter = 0;
	TIMSK |= TIMER_OVERFLOW_INTERUPT_ENABLE; //enable timer overflow interrupt.
}

void timer_stop(void){
	TCCR0 = CLOCK_SOURCE_DISABLED;
	TIMSK &= (unsigned char)~TIMER_OVERFLOW_INTERUPT_ENABLE;
}
	

#ifndef TIM0_OVF_vect
	#define TIM0_OVF_vect TIMER0_OVF_vect
#endif

ISR(TIM0_OVF_vect){
	TCNT0 += TIMER_INIT_VALUE;
		
	timer_interrupt_counter++;
	if (timer_interrupt_counter >= timer_interval){
		timer_interrupt_counter = 0;
		timer_handler();
	}
}



#endif